"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Query = void 0;
const lookup_enum_1 = require("./lookup.enum");
class Query {
    constructor(queryObject = {}) {
        this.queryObject = queryObject;
    }
    fieldContains(field, value, not = false) {
        this.queryObject[`${field}__${not ? lookup_enum_1.LookupFilter.NOT + '__' : ''}${lookup_enum_1.LookupFilter.CONTAINS}`] = value;
        return this;
    }
    fieldIn(field, elements, not = false) {
        this.queryObject[`${field}__${not ? lookup_enum_1.LookupFilter.NOT + '__' : ''}${lookup_enum_1.LookupFilter.IN}`] = elements.join(',');
        return this;
    }
    fieldEqual(field, value, not = false) {
        this.queryObject[`${field}${not ? '__' + lookup_enum_1.LookupFilter.NOT : ''}`] = value;
        return this;
    }
    fieldIsNull(field, not = false) {
        this.queryObject[`${field}__${not ? lookup_enum_1.LookupFilter.NOT + '__' : ''}${lookup_enum_1.LookupFilter.IS_NULL}`] = true;
        return this;
    }
    fieldGreaterThan(field, value, not = false) {
        this.queryObject[`${field}__${not ? lookup_enum_1.LookupFilter.NOT + '__' : ''}${lookup_enum_1.LookupFilter.GT}`] = value;
        return this;
    }
    fieldGreaterThanOrEqual(field, value, not = false) {
        this.queryObject[`${field}__${not ? lookup_enum_1.LookupFilter.NOT + '__' : ''}${lookup_enum_1.LookupFilter.GTE}`] = value;
        return this;
    }
    fieldLessThan(field, value, not = false) {
        this.queryObject[`${field}__${not ? lookup_enum_1.LookupFilter.NOT + '__' : ''}${lookup_enum_1.LookupFilter.LT}`] = value;
        return this;
    }
    fieldLessThanOrEqual(field, value, not = false) {
        this.queryObject[`${field}__${not ? lookup_enum_1.LookupFilter.NOT + '__' : ''}${lookup_enum_1.LookupFilter.LTE}`] = value;
        return this;
    }
    fieldStartsWith(field, value, not = false) {
        this.queryObject[`${field}__${not ? lookup_enum_1.LookupFilter.NOT + '__' : ''}${lookup_enum_1.LookupFilter.STARTS_WITH}`] = value;
        return this;
    }
    fieldEndsWith(field, value, not = false) {
        this.queryObject[`${field}__${not ? lookup_enum_1.LookupFilter.NOT + '__' : ''}${lookup_enum_1.LookupFilter.ENDS_WITH}`] = value;
        return this;
    }
    fieldBetween(field, firstValue, lastValue, not = false) {
        this.queryObject[`${field}__${not ? lookup_enum_1.LookupFilter.NOT + '__' : ''}${lookup_enum_1.LookupFilter.BETWEEN}`] = `${firstValue},${lastValue}`;
        return this;
    }
    orWhere(...subQueries) {
        if (!this.queryObject[lookup_enum_1.Operator.OR]) {
            this.queryObject[lookup_enum_1.Operator.OR] = [];
        }
        subQueries.forEach(subQuery => {
            this.queryObject[lookup_enum_1.Operator.OR].push(this.stringifyORCondition(subQuery.getRawObject()));
        });
        return this;
    }
    disablePagnination() {
        this.queryObject.pagination = false;
        return this;
    }
    enablePagnination() {
        this.queryObject.pagination = true;
        return this;
    }
    setPage(page) {
        this.queryObject.page = page;
        return this;
    }
    setLimit(limit) {
        this.queryObject.limit = limit;
        return this;
    }
    join(relations) {
        this.queryObject.join = relations.join(',');
        return this;
    }
    select(fields) {
        this.queryObject.select = fields.join(',');
        return this;
    }
    orderBy(field, by) {
        if (!this.queryObject.order)
            this.queryObject.order = '';
        this.queryObject.order += `${this.queryObject.order ? ',' : ''}${by === 'ASC' ? '^' : '-'}${field}`;
        return this;
    }
    toString() {
        if (!this.queryObject)
            return '';
        let result = '';
        for (const key in this.queryObject) {
            if (this.queryObject.hasOwnProperty(key)) {
                const value = this.queryObject[key];
                if (result)
                    result += '&';
                if (Array.isArray(value)) {
                    value.forEach((val, index) => {
                        result += `${key}=${val}${index !== value.length - 1 ? '&' : ''}`;
                    });
                }
                else {
                    result += `${key}=${value}`;
                }
            }
        }
        return result;
    }
    getRawObject() {
        return this.queryObject;
    }
    stringifyORCondition(query) {
        let result = '';
        for (const key in query) {
            if (query.hasOwnProperty(key)) {
                const value = query[key];
                result += `${key}:${value}|`;
            }
        }
        return result.substr(0, result.length - 1);
    }
}
exports.Query = Query;
